unit Jobs;

interface


uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;
 
type 
  
  TJobType = (jtNormal, jtSpecial, jtFull, jtEmpty);
  TMyEnum  = (meOne, meTwo, meThree, meFour, meFive);
  TMyEnums = set of TMyEnum;

  TJobHolder = class;

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH

  TJobLabel = class(TPersistent)
    private
      FJobName,
      FJob_ID              : string;
      FJobType             : TJobType;
    published
      property JobName     : string read FJobName write FJobName;
      property Job_ID      : string read FJob_ID write FJob_ID;
      property JobType     : TJobType read FJobType write FJobType;
  end;

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH

  TThirdLayer = class(TPersistent)
    private
      FS      : string;
      FI      : integer;
      FB      : boolean;
      FF      : double;
    published
      property S      : string read FS write FS;
      property I      : integer read FI write FI;
      property B      : boolean read FB write FB;
      property F      : double read FF write FF;
  end;

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH

  TSecondLayer = class(TPersistent)
    private
      FaString    : string;
      FaRect      : TRect;
      FaSet       : TMyEnums;
      FThirdLayer : TThirdLayer;
    public
      constructor Create;
    published
      property aRect      : TRect read FaRect write FaRect ;
      property aSet       : TMyEnums read FaSet write FaSet ;
      property aString    : string  read FaString write FaString;
      property ThirdLayer : TThirdLayer read FThirdLayer write FThirdLayer;
  end;

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH

  TJob = class(TCollectionItem)                  // First Layer
    private
      FaIndex       : integer;
      FaName        : string;
      FFull         : boolean;
      FSecondLayer  : TSecondLayer;
      function  GetSecondLayer: TSecondLayer;
      procedure SetSecondLayer(Value: TSecondLayer);
    public
      constructor Create(Collection: TCollection); override;
      procedure Assign(Source: TPersistent); override;
      destructor Destroy; override;
    published
      property aIndex       : integer read FaIndex write FaIndex;
      property aName        : string read FaName write FaName;
      property Full         : boolean read FFull write FFull;
      property SecondLayer  : TSecondLayer read GetSecondLayer write SetSecondLayer;
  end;

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH


  TJobs = class(TCollection)
    private
      FJobHolder            : TJobHolder;
      function GetItem(Index: Integer): TJob;
      procedure SetItem(Index: Integer; Value: TJob);
    protected
      function GetOwner: TPersistent; override;
    public
      constructor Create(JobHolder: TJobHolder);
      function Add: TJob;
      property Items[Index: Integer]: TJob read GetItem write SetItem; default;
  end;

//HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH


  TJobHolder = class(TComponent)
  private   //=======================================================================================
    FJobLabel   : TJobLabel;
    FJobs       : TJobs;

    procedure SetJobs(Value: TJobs);
  public    //=======================================================================================
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published //=======================================================================================
    property JobLabel:TJobLabel read FJobLabel write FJobLabel;
    property Jobs:TJobs  read FJobs write SetJobs;  
  end;






implementation




{===============================================================================
////////////////////////////////////////////////////////////////////////////////
///                                                                          ///
///                                                                          ///
                                  TSecondLayer
///                                                                          ///
///                                                                          ///
////////////////////////////////////////////////////////////////////////////////
===============================================================================}




{----------------------------Create--------------------------------------------

------------------------------------------------------------------------------}
constructor TSecondLayer.Create;
begin
  inherited Create;
  FThirdLayer := TThirdLayer.Create;
end;


{===============================================================================
////////////////////////////////////////////////////////////////////////////////
///                                                                          ///
///                                                                          ///
                                TJob
///                                                                          ///
///                                                                          ///
////////////////////////////////////////////////////////////////////////////////
===============================================================================}


{----------------------------Create-----------------------------------------

------------------------------------------------------------------------------}
constructor TJob.Create(Collection: TCollection);
begin         
  inherited Create(Collection);
end;

{----------------------------Destroy-------------------------------------------

------------------------------------------------------------------------------}
destructor TJob.Destroy;
begin
  FSecondLayer.Free;
  inherited  Destroy;
end;


{----------------------------Assign----------------------------------------

------------------------------------------------------------------------------}
procedure TJob.Assign(Source: TPersistent);
begin
  if Source is TJob then
  begin
    aIndex       := TJob(Source).aIndex;
    aName        := TJob(Source).aName;
    Full         := TJob(Source).Full;
    SecondLayer  := TJob(Source).SecondLayer;
    Exit;
  end;
  inherited Assign(Source);
end;


{--------------------------GetSecondLayer-------------------------------------

------------------------------------------------------------------------------}
function TJob.GetSecondLayer: TSecondLayer;
begin
  if FSecondLayer = nil then FSecondLayer := TSecondLayer.Create;
  Result := FSecondLayer;
end;


{---------------------------SetSecondLayer------------------------------------

------------------------------------------------------------------------------}
procedure TJob.SetSecondLayer(Value: TSecondLayer);
begin
  if Value = nil then
  begin
    FSecondLayer.Free;
    FSecondLayer := nil;
    Exit;
  end;
  FSecondLayer.Assign(Value);
end;




{===============================================================================
////////////////////////////////////////////////////////////////////////////////
///                                                                          ///
///                                                                          ///
                                    TJobs
///                                                                          ///
///                                                                          ///
////////////////////////////////////////////////////////////////////////////////
===============================================================================}





{----------------------------Create------------------------------------------

------------------------------------------------------------------------------}
constructor TJobs.Create(JobHolder: TJobHolder);
begin
  inherited Create(TJob);
  FJobHolder := JobHolder;
end;


{----------------------------Add---------------------------------------------

------------------------------------------------------------------------------}
function TJobs.Add: TJob;
begin
  Result := TJob(inherited Add);
end;

{----------------------------GetItem-------------------------------------------

------------------------------------------------------------------------------}
function TJobs.GetItem(Index: Integer): TJob;
begin
  Result := TJob(inherited GetItem(Index));
end;

{---------------------------SetItem------------------------------------------

------------------------------------------------------------------------------}
procedure TJobs.SetItem(Index: Integer; Value: TJob);
begin
  inherited SetItem(Index, Value);
end;


{---------------------------GetOwner-------------------------------------------

------------------------------------------------------------------------------}
function TJobs.GetOwner: TPersistent;
begin
  Result := FJobHolder;
end;







{===============================================================================
////////////////////////////////////////////////////////////////////////////////
///                                                                          ///
///                                                                          ///
                                  TJobHolder
///                                                                          ///
///                                                                          ///
////////////////////////////////////////////////////////////////////////////////
===============================================================================}




{-----------------------------Create------------------------------------------

------------------------------------------------------------------------------}
constructor TJobHolder.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FJobLabel   := TJobLabel.Create;
  FJobs       := TJobs.Create(Self);
end;

{----------------------------Destroy-----------------------------------------

------------------------------------------------------------------------------}
destructor TJobHolder.Destroy;
begin
  FJobLabel.Free;
  FJobs.Free;
  inherited Destroy;
end;

{-----------------------------SetJobs-------------------------------------------

------------------------------------------------------------------------------}
procedure TJobHolder.SetJobs(Value: TJobs);
begin
  FJobs.Assign(Value);
end;



end.